package example.hello;

import java.rmi.registry.Registry;
import java.rmi.registry.LocateRegistry;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

/**
 * Adapted from the "Hello World" example from Sun at 
 * http://java.sun.com/javase/6/docs/technotes/guides/rmi/hello/hello-world.html
 *
 * This is the actual concrete implementation of the Hello interface.
 */public class Server implements Hello {

	public Server() {}

	public static int REGISTRY_PORT = 1099;
	/**
	 * Concrete implementation of the method defined by the Hello interface.
	 */
	public String sayHello() throws RemoteException {
		return "Hello big world!";
	}

	private Registry registry;
		
	public void run(){
		try {
			//Try to get figure out this server's IP address and save it as the RMI server hostname.
			System.setProperty("java.rmi.server.hostname", java.net.InetAddress.getLocalHost().getHostAddress());

			// Set the URL to dynamically download code
			System.setProperty("java.rmi.server.codebase","http://"+System.getProperty("java.rmi.server.hostname")+"/");

			System.out.println("java.rmi.server.hostname: "+System.getProperty("java.rmi.server.hostname"));
			System.out.println("java.rmi.server.codebase: "+System.getProperty("java.rmi.server.codebase"));

		}
		catch(Exception e) {
			System.err.println("Error getting local host address: "+e+"\n");
		}


		// The problem is that LocateRegiistry.getRegistry will always return what appears to be a valid
		// registry value even if the registry doesn't exist.   This is because it technically returns
		// the stub to a registry, which may not actually be connected.  See the javadoc documentation for 
		// for LocateRegistry
		try {
			// First try to create a registry
			registry = LocateRegistry.createRegistry(REGISTRY_PORT);	
		}
		catch(RemoteException e){
			System.err.println("Could not create registry: "+e+"\n");
			System.err.println("Attempting to locate existing registry...");
			try {
				// Creating a new registry will fail if it already exists, so try to find it.
				registry = LocateRegistry.getRegistry(REGISTRY_PORT);
			}
			catch(RemoteException e2){
				System.err.println("Could not get registry: \n"+e2);
				System.exit(-1);  // exit the program.
			}		
		}
		

		try {
			// Set all RMI transactions to be printed out standard err.
			System.setProperty("java.rmi.server.logCalls","true");

			// instantiate the object to be served out to remote clients.
			Server obj = new Server();
			
			// Create a UnicastRemoteObject stub to be sent to the clients. 
			Hello stub = (Hello) UnicastRemoteObject.exportObject(obj, 0);

			// Bind the remote object's stub in the registry at the specified port
			// use rebind instead of bind so the program can be run multiple times with the same registry
			registry.rebind("Hello", stub);

			System.err.println("Server ready");
			
		} catch (Exception e) {
			System.err.println("Server exception: " + e.toString());
			e.printStackTrace();
		}
		
		// The following loop is just to keep the program from terminating
		// This loop will not be necessary once the server has a GUI 
		while(true){
			try {
				// Sleep this thread for 100 ms intervals to keep it from consuming resources
				Thread.sleep(100);  
			}
			catch(Exception e){}
		}
	}
	public static void main(String args[]) {  
		(new Server()).run();

	}
}
